"""
DBICKETT_LD48_4-16-2005_2.py
by Daniel Bickett
dbickett@gmail.com
http://www.heureusement.org/

NOTES:
	- This is by no means the final version.
	- Uploading it to demonstrate the shadow effect

CHANGELOG: (since last pre-release)
	- SHADOWS ADDED! :)
	- Preliminary code (alloc) is better organized
"""

import pygame
pygame.init()

import random
import Queue

# Constants
# - Directional
DIR_N = 101
DIR_E = 102
DIR_S = 103
DIR_W = 104

# Centralized Data: 'alloc' dict
# - Will Contain Graphics/Objects
# - Helps differentiate between locals and globals;
#   aids in orientation.

# screen related data
alloc = {}
alloc['screen_res'] = 608, 360
alloc['screen'] = pygame.display.set_mode( alloc['screen_res'] )

# load the images
alloc['image'] = {}
alloc['image']['background'] = pygame.image.load("background.gif").convert()
alloc['image']['cloud'] = pygame.image.load("cloud.gif").convert()
alloc['image']['shadow'] = pygame.image.load("cloud_shadow.gif").convert()
alloc['image']['shadow'].set_alpha( 130 )

for image in alloc['image'].itervalues():
	image.set_colorkey( ( 255 , 140 , 237 ) )

# rect objects
alloc['cloud_rect'] = alloc['image']['cloud'].get_rect()
alloc['ground_rect'] = pygame.Rect( 0 , 123 , 608 , 237 )
alloc['screen_rect'] = pygame.Rect( ( 0 , 0 ) , alloc['screen_res'] )
alloc['sky_rect'] = pygame.Rect( 0 , 0 , 608 , 123 )

class _Shadow:
	
	def __init__( self , initpos ):
		self.image = alloc['image']['shadow']
		self._setPos( initpos )

	def _setPos( self , pos ):
		self.pos = self.image.get_rect().move( pos )

class _Cloud:
	
	def __init__( self , initpos , speed , direction , shadowSpaceRect ):
		self.image = alloc['image']['cloud']
		shadowSpaceRect.left = initpos[0]
		self.shadow = _Shadow( shadowSpaceRect.topleft )
		self._setPos( initpos )
		self.speed = speed
		self.direction = direction

	def _setPos( self , pos ):
		self.pos = self.image.get_rect().move( pos )
		self.shadow._setPos( ( pos[0] , self.shadow.pos.top ) )

	def _step( self ):
		# check to see if this cloud needs to be recycled
		if ( self.direction == DIR_E and self.pos.left > alloc['screen_rect'].right ) or \
		( self.direction == DIR_W and self.pos.right < 0 ):
			# print "Recycling myself..."
			return False
		else:
			# step in the proper direction at the proper speed
			if self.direction == DIR_E:
				self.pos.right += self.speed
				self.shadow.pos.right += self.speed
			elif self.direction == DIR_W:
				self.pos.left -= self.speed
				self.shadow.pos.left -= self.speed
			return True

	def _blit( self ):
		alloc['screen'].blit( self.image , self.pos )
		alloc['screen'].blit( self.shadow.image , self.shadow.pos )

	def next( self ):
		if self._step():
			# print "Blitting"
			self._blit()
			return True
		else:
			return False

class Clouds:
	
	def __init__( self , cloud_num = 4 , min_speed = 1 , max_speed = 3 ):
		# NOTE: The cloud creation used to be done in a loop, but now we
		#       need to be more precise on account of the shadows. I'm
		#       fairly sure each level will only have four clouds, so I
		#       don't think this will become a problem.
		self.num = cloud_num
		self.min_speed = min_speed
		self.max_speed = max_speed
		# work with shadows
		shadowSpaceHeight = int( alloc['ground_rect'].bottom / 6 )
		self.shadspc = {}
		self.shadspc['A'] = pygame.Rect( alloc['ground_rect'].topleft , ( 608 , shadowSpaceHeight ) )
		self.shadspc['B'] = pygame.Rect( alloc['ground_rect'].left , self.shadspc['A'].top + shadowSpaceHeight , 608 , shadowSpaceHeight )
		self.shadspc['C'] = pygame.Rect( alloc['ground_rect'].left , self.shadspc['B'].top + shadowSpaceHeight , 608 , shadowSpaceHeight )
		self.shadspc['D'] = pygame.Rect( alloc['ground_rect'].left , self.shadspc['C'].top + shadowSpaceHeight , 608 , shadowSpaceHeight )
		# clouds' container
		self.clouds = {}
		self.clouds['A'] = self._genCloud( self.shadspc['A'] )
		self.clouds['B'] = self._genCloud( self.shadspc['B'] )
		self.clouds['C'] = self._genCloud( self.shadspc['C'] )
		self.clouds['D'] = self._genCloud( self.shadspc['D'] )

	def _genCred( self ):
		# direction
		direction = random.choice( ( DIR_W , DIR_E ) )
		# position
		coord = [0,0]
		if direction == DIR_E:
			coord[0] = - alloc['cloud_rect'].right
		else:
			coord[0] = alloc['sky_rect'].right + alloc['cloud_rect'].right
		coord[1] = random.randint( alloc['sky_rect'].top , alloc['sky_rect'].bottom - alloc['cloud_rect'].bottom )
		# speed
		speed = random.randint( self.min_speed , self.max_speed )
		# done generating.
		return direction, speed, coord
	
	def _genCloud( self , shadowSpace ):
		direction, speed, coord = self._genCred()
		return _Cloud( coord , speed , direction , shadowSpace )

	def next( self ):
		for cloud in self.clouds.itervalues():
			if not cloud.next():
				#print "Recycling cloud..."
				direction, speed, coord = self._genCred()
				cloud._setPos( coord )
				cloud.speed = speed
				cloud.direction = direction

CLOUDS = Clouds()

while True:
	# Blit the background
	alloc['screen'].blit( alloc['image']['background'] , ( 0 , 0 ) )
	# Event Loop
	for event in pygame.event.get():
		if event.type == pygame.QUIT:
			raise SystemExit
	# Cloud Logic
	CLOUDS.next()
	# Update the screen
	pygame.display.update()